'use strict';

import path from 'path';
import Knex from 'knex';
import Quture from 'quture-app';
import Config from 'quture-config';

export default class Database {

  /**
   * Create a new {Database} instance.
   *
   * @param {Quture} quture
   * @param {string} migrationCommand
   */
  constructor(quture) {
    const self = this;
    if (quture instanceof Quture) {
      this._quture = quture;
      this._config = new Config({
        configDir: path.join(this._quture.getConfig().getConfigDirectory(), 'database')
      });

      /**
       * Publish configuration file
       */
      this._quture.publishConfigurationFile(this._config, path.join(__dirname, 'config', 'connection.js'));

      /**
       * Configure knex
       */
      this._migrationsDir = this._config.get('connection.migrations', path.join(this._quture.getRootDirectory(), 'migrations'));

      let options = {
        client: this._config.get('connection.dialect', 'sqlite'),
        connection: {
          host: this._config.get('connection.host', '127.0.0.1'),
          user: this._config.get('connection.username', 'root'),
          password: this._config.get('connection.password', ''),
          database: this._config.get('connection.database', 'quture'),
          filename: this._config.get('connection.storage', path.join(this._quture.getStorageDirectory(), 'database.sqlite'))
        },
        migrations: {
          directory: this._migrationsDir,
          tableName: 'migrations'
        }
      };

      /**
       * Instantiate knex
       */
      this._knex = Knex(options);

      /**
       * Setup getQutureDatabase() method for allowing
       * the ability to retrieve this instance from
       * within migrations.
       */
      this._knex.getQutureDatabase = function() {
        return this;
      }

    } else {
      throw new Error('{quture} parameter must be an instance of {Quture}');
    }
  }

  /**
   * Retrieve the {Knex} instance.
   *
   * @returns {Knex}
   */
  knex() {
    return this._knex;
  }

  /**
   * Retrieve the migrations directory path.
   *
   * @return {string}
   */
  getMigrationsDirectory() {
    return this._migrationsDir;
  }

  /**
   * Allows the execution of a migration command.
   *
   * // TODO: actually make a command handler so that "plugins" can create
   * executable commands.
   *
   * @param {string} migrationCommand
   * @param {string} migrationName
   * @returns {Promise}
   */
  executeMigrationCommand(migrationCommand, migrationName) {
    if (typeof migrationCommand === 'string') {
      switch (migrationCommand.toLowerCase()) {
        case 'make':
          if (typeof migrationName !== 'string') {
            throw new Error('{migrationName} must be of type string');
          }

          return this._knex().migrate.make(migrationName);
        case 'migrate':
          return knex.migrate.latest();
        case 'rollback':
          return knex.migrate.rollback();
        case 'version':
          return knex.migrate.currentVersion();
        default:
          throw new Error('{migrationCommand} not recongnized');
      }
    } else {
      throw new Error('{migrationCommand} must be of type string');
    }
  }

  /**
   * Check if the migration file with the specified name
   * is already published to the migrations folder.
   *
   * @param {string} fileName
   * @returns {boolean}
   */
  isMigrationPublished(fileName) {
    return Quture.getUtilities().isFilePublished(fileName, this.getMigrationsDirectory());
  }

  /**
   * Publish the specified migration file into the
   * migration folder.
   *
   * @param {string} filePath
   */
  publishMigration(filePath) {
    return Quture.getUtilities().publishFile(filePath, this.getMigrationsDirectory());
  }

}
